import {
    CognitoUserPool,
    CognitoUser,
    AuthenticationDetails
} from 'amazon-cognito-identity-js';

import { MissingParameterError, UnavailableError } from '../../error';
import Session from '../../session';
import Logger from '../../logger';

const logger = new Logger('Cognito.UserPool');

/**
* Cognito User Pool client
* @class
*/
export default class UserPool {
    /**
    * @constructor
    * @param {object} config - User pool config
    <pre>
    {
        user_pool_id: ...,
        client_id: ...
    }
    </pre>
    */
    constructor(config) {
        if (!config) { throw new MissingParameterError('config'); }
        if (!config.user_pool_id) { throw new MissingParameterError('config.user_pool_id'); }
        if (!config.client_id) { throw new MissingParameterError('config.client_id'); }

        this.pool = new CognitoUserPool({
            UserPoolId: config.user_pool_id,
            ClientId: config.client_id
        });
    }

    signUp(username, password, attributes, callback) {
        if (!callback) { logger.warn('No callback for signUp'); }

        const attribute_list = Object.keys(attributes)
            .map(function(key) {
                return {
                    Name: key,
                    Value: attributes[key]
                };
            });

        this.pool.signUp(username, password, attribute_list, null, function(err, data) {
            if (err) {
                callback && callback(err);
            } else {
                const user = data.user;
                logger.debug('Sign up success: ' + user.getUsername());
                logger.debug(user);
                Session.set('username', user.getUsername());
                callback && callback(null, user);
            }
        });
    }

    signUpConfirm(code, callback) {
        const username = Session.get('username');
        if (!username) {
            callback && callback(new UnavailableError('Username', 'Get username then confirm'));
            return;
        }
        const user = _cognitoUser(username);
        user.confirmRegistration(code, true, function(err, data) {
            if (err) {
                callback && callback(err);
            } else {
                logger.debug(data);
                callback && callback(null, data);
            }
        });
    }

    signUpConfirmUser(username, code, callback) {
        Session.set('username', usernmae);
        this.signUpConfirm(code, callback);
    }

    signUpResendCode() {
        const username = (typeof arguments[0] === 'string')? arguments[0] : Session.get('username');
        const callback = (arguments.length > 0)? arguments[arguments.length-1] : null;
        if (!username) {
            callback && callback(new UnavailableError('Username', 'Get username then confirm'));
            return;
        }
        const user = _cognitoUser(username);
        user.resendConfirmationCode(function(err, data) {
            if (err) {
                callback && callback(err);
            } else {
                logger.debug(data);
                callback && callback(null, data);
            }
        });
    }

    signIn(username, password, callback) {
        var auth_details = new AuthenticationDetails({
            Username: username,
            Password: password
        });
        const auth_callbacks = {
            onSuccess: function(data) {
                logger.debug(data);
                callback && callback(null, data);
            },
            onFailure: function(err) {
                callback && callback(err);
            }
        }
        const user = _cognitoUser(username);
        user.authenticateUser(auth_details, auth_callbacks);
    }

    /*** private methods ***/
    _cognitoUser(username) {
        if (!this.pool) { throw new UnavailableError('CognitoPool'); }

        return new CognitoUser({
            Username: username,
            Pool: this.pool
        });
    }
};
